package eGuideDog::Browser::Widget;

use strict;
use warnings;
use eGuideDog::Browser::Const;

sub new;
sub detach;
sub press;
sub screen;
sub speech;
sub state;
sub callback;

sub new {
  my ($class, $callback, $caption) = @_;
  my $self = {screen => undef, # eGuideDog::Terminal::Buffer type
	      callback => $callback, # a sub routine entry
	      caption => $caption,
	      speech => undef,
	      speechs => undef,
	      state => '',
	     };
  bless $self, __PACKAGE__;
  return $self;
}

sub detach {
  my $self = shift;
  undef %{$self};
}

sub caption {
  my ($self, $caption) = @_;
  $self->{caption} = $caption if (defined $caption);
  return $self->{caption};
}

# screen buffer, derived from eGuideDog::Terminal::Buffer
sub screen {
  my ($self, $screen) = @_;
  $self->{screen} = $screen if (defined $screen);
  return $self->{screen};
}

sub speechs {
  my ($self, $speechs) = @_;
  if (defined $speechs) {
    $self->{speechs} = $speechs;
    return $speechs;
  } else {
    return $self->{speechs};
  }
}

# the buffer of speech will auto clear after read
sub speech {
  my ($self, $speech) = @_;
  if (defined $speech) {
    $self->{speech} = $speech;
  } elsif (defined $self->{speech}) {
    $speech = $self->{speech};
    $self->{speech} = undef;
    return $speech;
  } else {
    if ($self->{speechs} && $self->{speechs}->update) {
      $self->{speechs}->update(0);
      return $self->{speechs}->current_line;
    } else {
      return undef;
    }
  }
}

sub state {
  my ($self, $state) = @_;
  $self->{state} = $state if (defined $state);
  return $self->{state};
}

# callback function
sub callback {
  my ($self, $callback) = @_;
  $self->{callback} = $callback if (defined $callback);
  return $self->{callback};
}

# deal with key press
sub press {
  return eGuideDog::Browser::Const::THROW_KEY;
}

1;
